#ifndef __RSS_PARSER_H_
#define __RSS_PARSER_H_

#include <stdbool.h>
#include <libxml/parser.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Maximum length of title and URL */
#define RF_LEN		256
/* Maximum length of a description */
#define RF_DESCLEN	2048

/* Identifiers for all the RSS formats */
#define RSS_10		0
#define RSS_20		1
#define RSS_092		2
#define RSS_091		3
#define RSS_094		4
#define ATOM_03		5

/* Prefix spam with this string */
#define SPAMMSG		"*** SPAM ***"

/* RSS feed data structure */
typedef struct {
	char furl[RF_LEN];		/* feed URL */
	int V;					/* version */
	char title[RF_LEN];		/* title of the feed */
	char url[RF_LEN];		/* URL of the homepage */
	char desc[RF_DESCLEN];	/* description of the feed */
	char cfile[RF_LEN];		/* cache file */
	xmlDocPtr doc;			/* XML document */
	xmlNodePtr root;		/* XML root */
	xmlNodePtr cur;			/* pointer to the current XML item */
	int count;				/* Number of news items */
} feed;

/* RSS news item structure */
typedef struct {
	char title[RF_LEN];		/* title of the news item */
	char url[RF_LEN];		/* URL of the news item */
	char desc[RF_DESCLEN];	/* description of the news item */
	bool read;				/* read-status of the news item */
} newsitem;

/** Debugging output
@param point The error occured here
@return Error code
*/
int rss_err (char *point);
/** Initialize an RSS feed structure
@param fd User-allocated resulting feed structure
@param furl Fetch the feed from this URL
@return False on error, true otherwise
*/
bool rss_init (feed *fd, char *furl);
/** Initialize an RSS feed structure
@param fd User-allocated resulting feed structure
@param cfile Read the feed from this cache file
@return False on error, true otherwise
*/
bool rss_init_cache (feed *fd, char *cfile);
/** Get the next news item
@param fd The feed to read from
@param item User-allocated resulting news item structure
@return False on error, true otherwise
*/
bool rss_next (feed *fd, newsitem *item);
/** Reset current item to head of document
@param fd The feed to reset
@return False on error, true otherwise
*/
bool rss_reset (feed *fd);
/** Prepare the feed for reading the news items
@param fd The feed to prepare
@return False on error, true otherwise
*/
bool rss_begin (feed *fd);
/** Free all data structures in a feed (not the feed itself)
@param fd The feed to free
@return False on error, true otherwise
*/
bool rss_end (feed *fd);
/** Dump a news item to a file
@param file The file to dump to (stdout for screen)
@param item The news item structure
@return False on error, true otherwise
*/
bool rss_dump (FILE *file, newsitem item);
/** Dump a complete feed to a file
@param file The file to dump to (stdout for screen)
@param fd The newsfeed structure 
@return False on error, true otherwise
*/
bool rss_dump_feed (FILE *file, feed *fd);
/** Count items in a feed
@param fd The feed to work with
@param nospam Whether or not to count items considered as spam
@return Number of news items in the feed
*/
int rss_count (feed *fd, bool nospam);
/** Parse all items in a feed
@param fd The feed to read from
@param nospam Whether or not to include items considered as spam
@return Array of all news items
Return value must be free'd.
*/
newsitem *rss_parse (feed *fd, bool nospam);
/** Free all the newsitems 
@param fd The RSS feed
@param ret The newsitems
*/
void rss_unparse (feed *fd, newsitem *ret);
/** Get the next element in a feed
@param fd The feed to work with
@return Pointer to the next element 
*/
xmlNodePtr rss_nextptr (feed *fd);
/** Check an item's title for spam content
@param item The title to check
@return The new title, possibly prefixed by a spam identifier
Return value must be free'd.
*/
char *rss_spam_filter (char *item);
/** Search a XML subtree for a certain node
@param root Root of the XML subtree
@param sstr Name of the node to search for
@return Value of the node, empty string if the node was not found
Return value must be free'd.
*/
char *rss_searchval (xmlNodePtr root, char *sstr);
/** Return read-status of a given item
@param node Pointer to the item
@return Boolean read-status
*/
bool rss_readstat (xmlNodePtr node);
/** Find the channel entity in a RSS 0.94 feed
@param node Pointer to the root of the XML tree
@return Pointer to the first child of the channel entity
*/
xmlNodePtr rss094_chan(xmlNodePtr node);
/** Find the channel entity in a RSS 1.0 feed
@param node Pointer to the root of the XML tree
@return Pointer to the first child of the channel entity
*/
xmlNodePtr rss10_chan(xmlNodePtr node);
/** Find the channel entity in a RSS 2.0 (or compatible) feed
@param node Pointer to the root of the XML tree
@return Pointer to the first child of the channel entity
*/
xmlNodePtr rss20_chan(xmlNodePtr node);

#ifdef __cplusplus
}
#endif
#endif
